@@ -3,25 +3,25 @@ from collections import Counter
 import numpy as np
 import pandas as pd
 from dsbox.ml.outliers import fft_outliers, mad_outliers
-from sklearn.base import BaseEstimator, ClusterMixin
+from sklearn.base import BaseEstimator, ClusterMixin, OutlierMixin
 from sklearn.cluster import DBSCAN
 from sklearn.cluster import KMeans, MiniBatchKMeans
 from sklearn.covariance import EmpiricalCovariance, MinCovDet
 from sklearn.exceptions import NotFittedError
 from sklearn.gaussian_process import GaussianProcessRegressor
 from sklearn.mixture import BayesianGaussianMixture
-from sklearn.mixture.base import BaseMixture
-from sklearn.neighbors.kde import KernelDensity
+from sklearn.mixture._base import BaseMixture
+from sklearn.neighbors._kde import KernelDensity
 from sklearn.utils.validation import check_is_fitted, column_or_1d
 
-from scipy.stats import norm
+from scipy.stats import norm, median_absolute_deviation
 
 __author__ = "Vincent Levorato, Rémy Frenoy"
 __credits__ = "https://github.com/octo-technology/bdacore"
 __license__ = "Apache 2.0"
 
 
-class CovarianceOutliers(BaseEstimator):
+class CovarianceOutliers(BaseEstimator, OutlierMixin):
     """ Covariance wrapper outlier estimator.
     
     Encapsulates sklearn covariance estimator to extract outliers from Mahalanobis squared distance.
@@ -51,12 +51,8 @@ class CovarianceOutliers(BaseEstimator):
     
     >>> X = pd.DataFrame([1, 0, 0, 1, 10, 2, 115, 110, 32, 16, 2, 0, 15, 1])
     >>> cov_outliers = CovarianceOutliers(MinCovDet())
-    >>> cov_outliers.fit(X)
-    CovarianceOutliers(cov_estimator=MinCovDet(assume_centered=False, random_state=None, store_precision=True,
-         support_fraction=None),
-              threshold=None)
-    >>> outliers = cov_outliers.predict(X)
-    >>> outliers.values
+    >>> outliers = cov_outliers.fit_predict(X)
+    >>> outliers
     array([False, False, False, False, False, False,  True,  True, False,
            False, False, False, False, False])
     
@@ -130,7 +126,7 @@ class CovarianceOutliers(BaseEstimator):
         return probas > self.threshold
 
 
-class GaussianProcessOutliers(BaseEstimator):
+class GaussianProcessOutliers(BaseEstimator, OutlierMixin):
     """ Gaussian Process wrapper outlier estimator
 
     Encapsulates sklearn Gaussian Process Regressor estimator to extract outliers which should be
@@ -159,11 +155,7 @@ class GaussianProcessOutliers(BaseEstimator):
     >>> df = pd.DataFrame(data)
     
     >>> gp_outliers = GaussianProcessOutliers(GaussianProcessRegressor(alpha=0.95, normalize_y=True), n_samples=100)
-    >>> gp_outliers.fit(df)
-    GaussianProcessOutliers(gp_estimator=GaussianProcessRegressor(alpha=0.95, copy_X_train=True, kernel=None,
-                 n_restarts_optimizer=0, normalize_y=True,
-                 optimizer='fmin_l_bfgs_b', random_state=None),
-                n_samples=100)
+    >>> _ = gp_outliers.fit(df)
     >>> outliers = gp_outliers.predict(df, confidence=0.999)
     >>> outlier_positions = np.argwhere(outliers == np.amax(outliers)).flatten().tolist()
     >>> outlier_positions
@@ -197,8 +189,7 @@ class GaussianProcessOutliers(BaseEstimator):
 
         y = column_or_1d(X)
         x = np.arange(0, y.size)
-        df_x = pd.DataFrame(x)
-        self.gp_estimator.fit(df_x, y)
+        self.gp_estimator.fit(x.reshape(-1, 1), y)
 
         return self
 
@@ -240,7 +231,7 @@ class GaussianProcessOutliers(BaseEstimator):
         return outliers
 
 
-class KMeansOneClusterOutliers(BaseEstimator):
+class KMeansOneClusterOutliers(BaseEstimator, OutlierMixin):
     """ KMeans One Cluster Outliers estimator
     
     This estimator uses distance information to categorize outliers. In this case, KMeans algorithm is used,
@@ -250,7 +241,7 @@ class KMeansOneClusterOutliers(BaseEstimator):
     Parameters
     ----------
     
-    kmeans_estimator : KMeans, optional (default=KMeans(n_clusters=1, n_jobs=-1))
+    kmeans_estimator : KMeans, optional (default=KMeans(n_clusters=1))
     
     threshold : float (default=None)
         Used by predict method : if probability returned by predict_proba method is above this value, the element 
@@ -264,19 +255,14 @@ class KMeansOneClusterOutliers(BaseEstimator):
     
     >>> df = pd.DataFrame([1, 0, 0, 1, 10, 2, 115, 110, 32, 16, 2, 0, 15, 1])
     >>> kmoc_outliers = KMeansOneClusterOutliers()
-    >>> kmoc_outliers.fit(df)
-    KMeansOneClusterOutliers(kmeans_estimator=KMeans(algorithm='auto', copy_x=True, init='k-means++', max_iter=300,
-        n_clusters=1, n_init=10, n_jobs=-1, precompute_distances='auto',
-        random_state=None, tol=0.0001, verbose=0),
-                 threshold=None)
-    >>> outliers = kmoc_outliers.predict(df)
+    >>> outliers = kmoc_outliers.fit_predict(df)
     >>> outliers
     array([False, False, False, False, False, False,  True,  True, False,
            False, False, False, False, False])
     
     """
 
-    def __init__(self, kmeans_estimator=KMeans(n_clusters=1, n_jobs=-1), threshold=None):
+    def __init__(self, kmeans_estimator=KMeans(n_clusters=1), threshold=None):
         if not isinstance(kmeans_estimator, KMeans) and not isinstance(kmeans_estimator, MiniBatchKMeans):
             raise TypeError("Estimator must be a sklearn.cluster.KMeans or MiniBatchKMeans class")
 
@@ -345,7 +331,7 @@ class KMeansOneClusterOutliers(BaseEstimator):
         return probas > self.threshold
 
 
-class KMeansIterativeOneClusterOutliers(BaseEstimator):
+class KMeansIterativeOneClusterOutliers(BaseEstimator, OutlierMixin):
     """ KMeans Iterative One Cluster Outliers estimator
 
     This estimator uses iteratively KMeans One Cluster Outliers estimator to find outliers. It applies n times
@@ -376,7 +362,7 @@ class KMeansIterativeOneClusterOutliers(BaseEstimator):
 
     def __init__(self, kmoc_estimator=KMeansOneClusterOutliers(), n_iterations=5):
         if not isinstance(kmoc_estimator, KMeansOneClusterOutliers):
-            raise TypeError("Estimator must be a bdacore.outliers.KMeansOneClusterOutliers class")
+            raise TypeError("Estimator must be a dsbox.ml.outliers.KMeansOneClusterOutliers class")
 
         self.kmoc_estimator = kmoc_estimator
         self.n_iterations = n_iterations
@@ -418,7 +404,7 @@ class KMeansIterativeOneClusterOutliers(BaseEstimator):
         return outlier_iter
 
 
-class GMMOutliers(BaseEstimator):
+class GMMOutliers(BaseEstimator, OutlierMixin):
     """ Gaussian mixture model Outliers estimator
     
     This estimator uses density information to categorize outliers. In this case, GMM algorithm is used,
@@ -442,16 +428,7 @@ class GMMOutliers(BaseEstimator):
     
     >>> df = pd.DataFrame([1, 0, 0, 1, 10, 2, 115, 110, 32, 16, 2, 0, 15, 1])
     >>> gmm_outliers = GMMOutliers()
-    >>> gmm_outliers.fit(df)
-    GMMOutliers(gmm_estimator=BayesianGaussianMixture(covariance_prior=None, covariance_type='full',
-                degrees_of_freedom_prior=None, init_params='kmeans',
-                max_iter=100, mean_precision_prior=None, mean_prior=None,
-                n_components=1, n_init=1, random_state=None, reg_covar=1e-06,
-                tol=0.001, verbose=0, verbose_interval=10, warm_start=False,
-                weight_concentration_prior=None,
-                weight_concentration_prior_type='dirichlet_process'),
-          threshold=None)
-    >>> outliers = gmm_outliers.predict(df)
+    >>> outliers = gmm_outliers.fit_predict(df)
     >>> outliers
     array([False, False, False, False, False, False,  True,  True, False,
            False, False, False, False, False])
@@ -522,7 +499,7 @@ class GMMOutliers(BaseEstimator):
         return probas > self.threshold
 
 
-class ClusteringOutliers(BaseEstimator):
+class ClusteringOutliers(BaseEstimator, OutlierMixin):
     """ Clustering model Outliers estimator
 
     This estimator uses clustering information to categorize outliers. Depending on the strategy, this
@@ -567,15 +544,7 @@ class ClusteringOutliers(BaseEstimator):
     
     >>> df = pd.DataFrame([1, 0, 0, 1, 10, 2, 115, 110, 32, 16, 2, 0, 15, 1])
     >>> clustering_outliers = ClusteringOutliers(cluster_estimator=KMeans(n_clusters=2), strategy='size')
-    >>> clustering_outliers.fit(df)
-    ClusteringOutliers(cluster_estimator=KMeans(algorithm='auto', copy_x=True, init='k-means++', max_iter=300,
-        n_clusters=2, n_init=10, n_jobs=1, precompute_distances='auto',
-        random_state=None, tol=0.0001, verbose=0),
-              kernel_density=KernelDensity(algorithm='auto', atol=0, bandwidth=0.2, breadth_first=True,
-           kernel='epanechnikov', leaf_size=40, metric='euclidean',
-           metric_params=None, rtol=0),
-              strategy='size', threshold=None)
-    >>> outliers = clustering_outliers.predict(df)
+    >>> outliers = clustering_outliers.fit_predict(df)
     >>> outliers
     array([False, False, False, False, False, False,  True,  True, False,
            False, False, False, False, False])
@@ -701,15 +670,18 @@ class ClusteringOutliers(BaseEstimator):
         return probas > self.threshold
 
 
-class MADOutliers(BaseEstimator):
+class MADOutliers(BaseEstimator, OutlierMixin):
     """  Median Absolute Deviation outliers estimator
     
-    A simple scikit wrapper using bdacore.outliers.utils.mad_outliers method. See documentation for
+    A simple scikit wrappering method used in dsbox.ml.outliers.utils.mad_outliers. See documentation for
     further purpose.
     
     
     Parameters
     ----------
+
+    window: int, optional (default=None)
+        window size for MAD calculation, if None, window is equal to length of dataset
     
     cutoff : int, optional (default=2)
         amount of times residuals relative to the median exceed the ratio to the MAD
@@ -720,17 +692,18 @@ class MADOutliers(BaseEstimator):
     >>> import pandas as pd 
     >>> from dsbox.ml.outliers import MADOutliers
     
-    >>> df = pd.DataFrame([1, 0, 0, 1, 10, 2, 115, 110, 32, 16, 2, 0, 15, 1])
+    >>> df = pd.DataFrame({'values': [1, 0, 0, 1, 10, 2, 115, 110, 32, 16, 2, 0, 15, 1]})
     >>> mad_outliers = MADOutliers()
     >>> outliers = mad_outliers.fit_predict(df)
-    >>> outliers.values
+    >>> outliers
     array([False, False, False, False,  True, False,  True,  True,  True,
-            True, False, False,  True, False])
+           False, False, False,  True, False])
     
     
     """
 
-    def __init__(self, cutoff=2, threshold=None):
+    def __init__(self, window=None, cutoff=2, threshold=None):
+        self.window = window
         self.cutoff = cutoff
         self.threshold = threshold
 
@@ -740,9 +713,7 @@ class MADOutliers(BaseEstimator):
         
         Parameters
         ----------
-        X : array-like, shape = [n_samples, n_features]
-          Training data, where n_samples is the number of samples
-          and n_features is the number of features.
+        X : pandas DataFrame
         y : not used, present for API consistence purpose.
         
         Returns
@@ -750,9 +721,16 @@ class MADOutliers(BaseEstimator):
         self : object
           Returns self.
         """
-        self.X_ = pd.DataFrame(columns=X.columns)
+        if self.window is None:
+            self.window = len(X)
+        self.X_ = pd.DataFrame()
         for column in X.columns:
-            self.X_[column] = mad_outliers(X[column], cutoff=self.cutoff)
+            self.X_['mad_{}'.format(column)] = X[column].rolling(self.window, min_periods=1).apply(
+                median_absolute_deviation, raw=False)
+            self.X_['median_{}'.format(column)] = X[column].rolling(self.window, min_periods=1).median()
+            self.X_[column] = (np.abs(X[column] - self.X_['median_{}'.format(column)]) / self.X_[
+                'mad_{}'.format(column)]) > self.cutoff
+            self.X_ = self.X_.drop(['mad_{}'.format(column), 'median_{}'.format(column)], axis=1)
 
         return self
 
@@ -762,7 +740,7 @@ class MADOutliers(BaseEstimator):
 
         Parameters
         ----------
-        X : array-like, shape = (n_samples, n_features)
+        X : not used, present for API consistence purpose.
 
         Returns
         -------
@@ -770,33 +748,30 @@ class MADOutliers(BaseEstimator):
 
         """
 
-        return self.X_.mean(axis=1)
+        return self.X_.mean(axis=1).values
 
     def predict(self, X):
         """
-        Returns a boolean tag for each element to be an outlier. It takes predict_proba method, 
+        Returns a boolean tag for each element to be an outlier. It takes predict_proba method,
         and checks if it exceeds the threshold attribute.
 
         Parameters
         ----------
-        X : array-like, shape = (n_samples, n_features)
+        X : not used, present for API consistence purpose.
 
         Returns
         -------
         Boolean array with outlier tag
         """
+
         probas = self.predict_proba(X)
 
         if self.threshold is None:
             self.threshold = np.mean(probas)
         return probas > self.threshold
 
-    def fit_predict(self, X):
-        self.fit(X)
-        return self.predict(X)
 
-
-class FFTOutliers(BaseEstimator):
+class FFTOutliers(BaseEstimator, OutlierMixin):
     """  Fast Fourier Transformation outliers estimator
 
     A simple scikit wrapper using bdacore.outliers.utils.fft_outliers method. See documentation for
@@ -892,7 +867,3 @@ class FFTOutliers(BaseEstimator):
         if self.threshold is None:
             self.threshold = np.mean(probas)
         return probas > self.threshold
-
-    def fit_predict(self, X):
-        self.fit(X)
-        return self.predict(X)
